import { Card, Checkbox, Col, Row, Select, Spin } from "antd";
import { useEffect, useRef, useState } from "react";
import { Constants, myFetch } from "../../utils";
import { useTranslation } from "react-i18next";
import { Virtuoso } from "react-virtuoso";

export default function LogCard({
  type,
  title,
  customResult,
  logApiAddress = Constants.LOG_API_ADDRESS,
}) {
  const { t } = useTranslation();
  const [checkboxInfoChecked, setCheckboxInfoChecked] = useState(true);
  const [checkboxDebugChecked, setCheckboxDebugChecked] = useState(true);
  const [checkboxWarningChecked, setCheckboxWarningChecked] = useState(true);
  const [checkboxErrorChecked, setCheckboxErrorChecked] = useState(true);
  const virtuosoRef = useRef();

  const [loadingSpinner, setLoadingSpinner] = useState(true);
  const [availableDates, setAvailableDates] = useState([]);
  const [selectedDate, setSelectedDate] = useState("");
  const [logs, setLogs] = useState([]);

  const sseEventSource = useRef(null);

  useEffect(() => {
    if (type === "") return;

    // init request to get all available dates
    myFetch(`s/${type}`, "GET", null, {}, 0, logApiAddress)
      .then((data) => {
        let dates = [];

        if (data.length === 0) return;

        dates = data.map((date) => {
          return {
            value: date,
            label: date,
          };
        });

        setAvailableDates(dates);
        setSelectedDate(dates[0].value);

        if (selectedDate === dates[0].value) {
          loadLogs(dates[0].value);
        }
      })
      .catch((errStatus) => {
        if (errStatus === 422) {
          setLoadingSpinner(false);
        }
      });
  }, [type]);

  const loadLogs = (date) => {
    setLoadingSpinner(true);

    if (date === "") return;

    // fetch logs
    myFetch(`s/${type}?d=${date}`, "GET", null, {}, 0, logApiAddress)
      .then((data) => {
        setLoadingSpinner(false);

        // data is undefined when there are no logs for the selected date (e.g. log was deleted)
        if (data === undefined) {
          setLogs([]);
          return;
        }

        setLogs(data);
      })
      .catch((errStatus) => console.log("error", errStatus));
  };

  useEffect(() => loadLogs(selectedDate), [selectedDate]);

  useEffect(() => {
    if (type === "" || selectedDate === "") return;

    const currentDate = new Date();

    const today = `${currentDate.getDate()}-${
      currentDate.getMonth() + 1
    }-${currentDate.getFullYear()}`;

    // only open the sse connection if the selected date is today
    if (selectedDate === today) {
      sseEventSource.current = new EventSource(
        `${logApiAddress}/sse/${type}/${selectedDate}`
      );

      sseEventSource.current.onmessage = (event) => {
        const data = JSON.parse(event.data);

        setLogs((prevLogs) => {
          const newLogs = [...prevLogs];

          data.forEach((log) => {
            newLogs.push(log);
          });

          return newLogs;
        });
      };

      sseEventSource.current.onerror = (event) =>
        console.log("sse error", event);
      /*
      sseEventSource.current.onopen = (event) => console.log("sse open", event);

      sseEventSource.current.onclose = (event) =>
        console.log("sse close", event); */

      return () => sseEventSource.current.close();
    }
  }, [type, selectedDate]);

  // {t: "D", m: []}
  // if type of next log is equal to the type of the last log, then add it to the last log otherwise create a new log
  const testLogs = () => {
    const result = [];

    let lastLogType = "";

    logs.forEach((log) => {
      const logType = log.charAt(0);

      if (
        logType === "I" ||
        logType === "D" ||
        logType === "W" ||
        logType === "E"
      ) {
        lastLogType = logType;
      } else if (log.match(/(\d{2}:\d{2}:\d{2})/)) {
        // check if log starts with time
        lastLogType = "404";
        log = "[TYPE NOT PROVIDED] " + log;
      }

      if (result.length === 0) {
        result.push({ t: lastLogType, m: [log] });
      } else {
        if (lastLogType === result[result.length - 1].t) {
          result[result.length - 1].m.push(log);
        } else {
          result.push({ t: lastLogType, m: [log] });
        }
      }
    });

    // condition check for filtering

    const filteredResult = result.filter((log) => {
      return (
        (checkboxInfoChecked && log.t === "I") ||
        (checkboxDebugChecked && log.t === "D") ||
        (checkboxWarningChecked && log.t === "W") ||
        (checkboxErrorChecked && log.t === "E") ||
        log.t === "404"
      );
    });

    return filteredResult;
  };

  /*
  const filteredLogs = logs.filter((log) => {
    const logType = log.charAt(0);

    // some other logs which have no type defined
    if (
      logType !== "I" &&
      logType !== "D" &&
      logType !== "W" &&
      logType !== "E"
    ) {
      return true;
    }

    return (
      (checkboxInfoChecked && logType === "I") ||
      (checkboxDebugChecked && logType === "D") ||
      (checkboxWarningChecked && logType === "W") ||
      (checkboxErrorChecked && logType === "E")
    );
  }); */

  let lastLogType = "";

  return (
    <Card
      title={
        <Row>
          <Col xs={24} xl={15}>
            {title}
          </Col>

          <Col xs={24} xl={9}>
            <Row xs={24} justify="end" align="middle">
              <Col>
                <Checkbox
                  disabled={customResult !== undefined}
                  checked={checkboxInfoChecked}
                  onChange={(e) => setCheckboxInfoChecked(e.target.checked)}
                >
                  {t("logCard.card.checkbox.info")}
                </Checkbox>
              </Col>
              <Col>
                <Checkbox
                  disabled={customResult !== undefined}
                  checked={checkboxDebugChecked}
                  onChange={(e) => setCheckboxDebugChecked(e.target.checked)}
                >
                  {t("logCard.card.checkbox.debug")}
                </Checkbox>
              </Col>
              <Col>
                <Checkbox
                  disabled={customResult !== undefined}
                  checked={checkboxWarningChecked}
                  onChange={(e) => setCheckboxWarningChecked(e.target.checked)}
                >
                  {t("logCard.card.checkbox.warning")}
                </Checkbox>
              </Col>
              <Col>
                <Checkbox
                  disabled={customResult !== undefined}
                  checked={checkboxErrorChecked}
                  onChange={(e) => setCheckboxErrorChecked(e.target.checked)}
                >
                  {t("logCard.card.checkbox.error")}
                </Checkbox>
              </Col>

              <Col>
                <Select
                  disabled={customResult !== undefined}
                  style={{ width: 120 }}
                  options={availableDates}
                  value={selectedDate}
                  onChange={(value) => setSelectedDate(value)}
                />
              </Col>
            </Row>
          </Col>
        </Row>
      }
      style={{
        height: "90vh",
        overflow: "hidden",
        overflowWrap: "break-word",
        width: "100%",
      }}
    >
      {customResult === undefined ? (
        <>
          {!loadingSpinner && type !== "" ? (
            <div style={{ height: "80vh", paddingRight: 6 }}>
              <Virtuoso
                ref={virtuosoRef}
                initialTopMostItemIndex={logs.length - 1}
                data={testLogs()}
                itemContent={(_, logData) => {
                  let items = [];

                  for (let i = 0; i < logData.m.length; i++) {
                    const logType = logData.t;

                    let style = {};
                    let color = "";

                    if (
                      logType === "I" ||
                      logType === "D" ||
                      logType === "W" ||
                      logType === "E"
                    ) {
                      lastLogType = logType;
                    }

                    if (
                      checkboxInfoChecked &&
                      lastLogType === "I" &&
                      logType === "I"
                    ) {
                      color = "#44bd32";
                    }

                    if (
                      checkboxDebugChecked &&
                      lastLogType === "D" &&
                      logType === "D"
                    ) {
                      color = "#9b59b6";
                    }

                    if (
                      checkboxWarningChecked &&
                      lastLogType === "W" &&
                      logType === "W"
                    ) {
                      color = "#e67e22";
                    }

                    if (
                      (checkboxErrorChecked &&
                        lastLogType === "E" &&
                        logType === "E") ||
                      logType === "404"
                    ) {
                      color = "#e74c3c";
                    }

                    if (color === "") {
                      color = "#2980b9"; // no match
                    }

                    style = { padding: "1px", color, whiteSpace: "pre-line" };

                    items.push(
                      <div key={i}>
                        <span style={style}>{logData.m[i]}</span>
                        <br />
                      </div>
                    );
                  }

                  return items;
                }}
                // data={filteredLogs}
                /*itemContent={(_, log) => {
                  const logType = log.charAt(0);
                  let style = {};
                  let color = "";

                  if (
                    logType === "I" ||
                    logType === "D" ||
                    logType === "W" ||
                    logType === "E"
                  ) {
                    lastLogType = logType;
                  }

                  if (checkboxInfoChecked && lastLogType === "I") {
                    color = "#44bd32";
                  } else if (checkboxDebugChecked && lastLogType === "D") {
                    color = "#9b59b6";
                  } else if (checkboxWarningChecked && lastLogType === "W") {
                    color = "#e67e22";
                  } else if (checkboxErrorChecked && lastLogType === "E") {
                    color = "#e74c3c";
                  } else {
                    color = "#2980b9"; // no match
                  }

                  style = { padding: "1px", color, whiteSpace: "pre-line" };

                  return <span style={style}>{log}</span>;
                }}*/
                followOutput={true}
              />
            </div>
          ) : (
            <div
              style={{
                display: "flex",
                justifyContent: "center",
                alignItems: "center",
                height: "80vh",
              }}
            >
              <Spin size="large"></Spin>
            </div>
          )}
        </>
      ) : (
        <div
          style={{
            display: "flex",
            justifyContent: "center",
            alignItems: "center",
            height: "80vh",
          }}
        >
          {customResult}
        </div>
      )}
    </Card>
  );
}
