import {
  Button,
  Col,
  Form,
  Popconfirm,
  Result,
  Row,
  Tag,
  Typography,
} from "antd";
import { useEffect, useState } from "react";
import GroupTasksViewModal from "./GroupTasksViewModal";
import GroupTypeSelectionModal from "./GroupTypeSelectionModal";
import GroupTaskTableList from "./GroupTasksTableList";
import {
  Constants,
  hasPermission,
  myFetch,
  wsConnectionCustomEventName,
} from "../../../utils";
import { useTranslation } from "react-i18next";
import { ReloadOutlined } from "@ant-design/icons";
import { useGroupTasksContext } from "../../../Contexts/GroupTasksContext";
import { useAppContext } from "../../../Contexts/AppContext";
import { useWebSocketContext } from "../../../Contexts/WebSocketContext";
import { useParams } from "react-router-dom";
import MyPagination from "../../../Components/MyPagination";
import { SentMessagesCommands } from "../../../Handlers/WebSocketMessageHandler";

export default function GroupTasks({ isGroupTasksViewModalOpen }) {
  const webSocketContext = useWebSocketContext();
  const appContext = useAppContext();
  const groupTasksContext = useGroupTasksContext();
  const { paramCategory } = useParams();

  const [isGroupTypeSelectionModalOpen, setIsGroupTypeSelectionModalOpen] =
    useState(false);
  const [currentCategoryGroup, setCurrentCategoryGroup] = useState([]);
  const [currentSelectedModalGroupType, setCurrentSelectedModalGroupType] =
    useState();
  const { t } = useTranslation();

  const showGroupTypeSelectionModal = (categoryGroup) => {
    setCurrentCategoryGroup(categoryGroup);
    setIsGroupTypeSelectionModalOpen(true);
    setCurrentSelectedModalGroupType(null);
  };

  const onInstallGlobalPythonPackages = () => {
    webSocketContext.SendSocketMessage(
      SentMessagesCommands.GroupTasksInstallGlobalPythonPackages,
      {}
    );
  };

  const fetchGroupTasks = (page) => {
    myFetch(`/grouptasks/${paramCategory}?page=${page}`, "GET").then((data) => {
      groupTasksContext.setTotalPages(data.TotalPages);

      groupTasksContext.setCategoryGroup(data.CategoryGroup);
      groupTasksContext.setGroupTasks(data.GroupTasks);
    });
  };

  const onPaginationChange = (page) => {
    groupTasksContext.setPaginationPage(page);
    groupTasksContext.paginationPageRef.current = page;
  };

  useEffect(() => {
    // initially previousParamCategory.current is null on first render
    if (groupTasksContext.previousParamCategory.current === null) {
      groupTasksContext.previousParamCategory.current = paramCategory;
      return;
    }

    if (paramCategory !== groupTasksContext.previousParamCategory.current) {
      groupTasksContext.previousParamCategory.current = paramCategory;

      if (groupTasksContext.paginationPage === 1) {
        // if we are on page 1, we need to fetch the new category group
        fetchGroupTasks(1);
      } else {
        // if we are not on page 1, we need to reset the page to 1 and this will trigger the useEffect below
        onPaginationChange(1);
      }
    } else {
      fetchGroupTasks(groupTasksContext.paginationPage);
    }
  }, [paramCategory]);

  useEffect(() => {
    const groupTasksRequest = () =>
      fetchGroupTasks(groupTasksContext.paginationPage);

    groupTasksRequest();

    const handleGroupTasksRequest = () => groupTasksRequest();

    document.addEventListener(
      wsConnectionCustomEventName,
      handleGroupTasksRequest
    );

    return () =>
      document.removeEventListener(
        wsConnectionCustomEventName,
        handleGroupTasksRequest
      );
  }, [groupTasksContext.paginationPage]);

  useEffect(
    () => groupTasksContext.setSelectInputs({}),
    [currentSelectedModalGroupType]
  );

  return (
    <>
      {groupTasksContext.categoryGroup.Category === "" ||
      groupTasksContext.categoryGroup.groups === null ? (
        <Result status="404" title={t("groupTasks.categoryGroups.length0")} />
      ) : (
        <>
          <Row>
            <Col
              xs={24}
              sm={{ span: 10, offset: 14 }}
              md={{ span: 8, offset: 16 }}
            >
              {hasPermission(
                appContext.userPermissions,
                Constants.PERMISSIONS.GROUP_TASKS.INSTALL_GLOBAL_PYTHON_PACKAGES
              ) && (
                <div>
                  <Popconfirm
                    placement="top"
                    okText={t("common.button.confirm")}
                    cancelText={t("common.button.cancel")}
                    title={t(
                      "groupTasks.button.installGlobalPythonPackages.popover.title"
                    )}
                    onConfirm={() => onInstallGlobalPythonPackages()}
                  >
                    <Button icon={<ReloadOutlined />} block>
                      {t("groupTasks.button.installGlobalPythonPackages.title")}
                    </Button>
                  </Popconfirm>
                </div>
              )}
            </Col>
          </Row>

          {Object.keys(groupTasksContext.categoryGroup).length > 0 && (
            <>
              <GroupTaskTableList
                categoryGroup={groupTasksContext.categoryGroup}
                showGroupTypeSelectionModal={showGroupTypeSelectionModal}
                groupTasks={groupTasksContext.groupTasks}
              />

              <MyPagination
                paginationPage={groupTasksContext.paginationPage}
                setPaginationPage={(page) => onPaginationChange(page)}
                totalPages={groupTasksContext.totalPages}
              />
            </>
          )}
        </>
      )}

      <GroupTypeSelectionModal
        isOpen={isGroupTypeSelectionModalOpen}
        setIsOpen={setIsGroupTypeSelectionModalOpen}
        categoryGroup={currentCategoryGroup}
        currentSelectedModalGroupType={currentSelectedModalGroupType}
        setCurrentSelectedModalGroupType={setCurrentSelectedModalGroupType}
      />

      <GroupTasksViewModal isOpen={isGroupTasksViewModalOpen} />
    </>
  );
}

export function MyFormItem({
  children,
  t,
  style,
  labelDisplayName,
  isGlobal,
  name,
}) {
  return (
    <Form.Item
      style={style}
      required
      name={name}
      label={
        labelDisplayName !== undefined && (
          <MyFormItemLabel
            t={t}
            displayName={labelDisplayName}
            isGlobal={isGlobal}
          />
        )
      }
    >
      {children}
    </Form.Item>
  );
}

export function MyFormItemLabel({ t, displayName, isGlobal }) {
  return (
    <>
      {displayName === undefined || displayName === "" ? (
        <Typography.Text type="danger">DisplayName is missing</Typography.Text>
      ) : (
        displayName
      )}

      {isGlobal && (
        <Tag style={{ marginLeft: 6 }} color="purple">
          {t("groupTasks.tag.global")}
        </Tag>
      )}
    </>
  );
}
