import {
  Button,
  Card,
  Col,
  Form,
  Input,
  Popconfirm,
  Popover,
  Row,
  Select,
  Space,
  Table,
  Tooltip,
  Typography,
  Upload,
  notification,
} from "antd";
import { useEffect, useRef, useState } from "react";
import {
  Constants,
  EncodeStringToBase64,
  FormatDatetime,
  getConnectionStatusItem,
  getUserSessionFromLocalStorage,
  hasPermission,
  isEmailValid,
  myFetch,
  myFetchContentType,
  wsConnectionCustomEventName,
} from "../../utils";
import { Link } from "react-router-dom";
import { Trans, useTranslation } from "react-i18next";
import {
  DisconnectOutlined,
  FileTextOutlined,
  InfoCircleOutlined,
  KeyOutlined,
  LogoutOutlined,
  NotificationOutlined,
} from "@ant-design/icons";
import { MyUserAvatar } from "../../Components/MyAvatar";
import { useUserProfileContext } from "../../Contexts/UserProfileContext";
import { useWebSocketContext } from "../../Contexts/WebSocketContext";
import { useAppContext } from "../../Contexts/AppContext";
import { useSideBarContext } from "../../Contexts/SideBarContext";
import { SentMessagesCommands } from "../../Handlers/WebSocketMessageHandler";
import {
  MyCopyIcon,
  MyReloadIcon,
  MyShowHiddenIcon,
} from "../../Components/MyIcon";

export default function UserProfile({ userSession, setUserSession }) {
  const webSocketContext = useWebSocketContext();
  const appContext = useAppContext();
  const sideBarContext = useSideBarContext();
  const userProfileContext = useUserProfileContext();
  const [notificationApi, notificationContextHolder] =
    notification.useNotification();
  const { t, i18n } = useTranslation();

  const [username, setUsername] = useState(Constants.LOADING);
  const [email, setEmail] = useState(Constants.LOADING);
  const [oldPassword, setOldPassword] = useState("");
  const [newPassword, setNewPassword] = useState("");
  const [repeatedNewPassword, setRepeatedNewPassword] = useState("");
  const [newApiKeyName, setNewApikeyName] = useState("");
  const [showApiKeyPassword, setShowApiKeyPassword] = useState(false);
  const [
    telegramNotificationSubscribedStatus,
    setTelegramNotificationSubscribedStatus,
  ] = useState(null);

  const statusInterval = useRef(null);
  const telegramVerifyCodeRequestTime = useRef(null);

  const getSessionTableColumns = () => {
    return [
      {
        title: t("userProfile.column.userAgent"),
        dataIndex: "userAgent",
        key: "userAgent",
      },
      {
        title: t("userProfile.column.connectionStatus"),
        dataIndex: "connectionStatus",
        key: "connectionStatus",
      },
      {
        title: t("userProfile.column.lastUsed"),
        dataIndex: "lastUsed",
        key: "lastUsed",
      },
      {
        title: t("userProfile.column.expiresAt"),
        dataIndex: "expiresAt",
        key: "expiresAt",
      },
      {
        title: t("userProfile.column.action.title"),
        dataIndex: "action",
        key: "action",
        render: (_, record) => {
          return (
            <Space size="middle">
              <Link
                to="#"
                onClick={() => myFetch(`/user/session/${record.key}`, "DELETE")}
              >
                {t("userProfile.column.action.signOut")}
              </Link>
            </Space>
          );
        },
      },
    ];
  };

  const getApiKeyTableColumns = () => {
    return [
      {
        title: t("userProfile.column.name"),
        dataIndex: "name",
        key: "name",
      },
      {
        title: t("userProfile.column.token"),
        dataIndex: "token",
        key: "token",
        render: (text) => (
          <Space>
            {showApiKeyPassword ? text : "XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX"}

            <MyShowHiddenIcon
              isHidden={showApiKeyPassword}
              setIsHidden={setShowApiKeyPassword}
            />

            <MyCopyIcon text={text} notificationApi={notificationApi} />
          </Space>
        ),
      },
      {
        title: t("userProfile.column.usageCount"),
        dataIndex: "usageCount",
        key: "usageCount",
      },

      {
        title: t("userProfile.column.lastUsed"),
        dataIndex: "lastUsed",
        key: "lastUsed",
      },
      {
        title: t("userProfile.column.createdAt"),
        dataIndex: "createdAt",
        key: "createdAt",
      },
      {
        title: t("userProfile.column.action.title"),
        dataIndex: "action",
        key: "action",
        render: (_, record) => (
          <Space size="middle">
            <Popconfirm
              placement="left"
              title={t("userProfile.column.deleteApiKey.popconfirm.title")}
              description={t(
                "userProfile.column.deleteApiKey.popconfirm.description"
              )}
              okText={t("common.button.delete")}
              cancelText={t("common.button.cancel")}
              onConfirm={() =>
                webSocketContext.SendSocketMessage(
                  SentMessagesCommands.DeleteUserApiKey,
                  { Id: record.key }
                )
              }
            >
              <Link to="#">{t("userProfile.column.deleteApiKey.link")}</Link>
            </Popconfirm>
          </Space>
        ),
      },
    ];
  };

  const getApiKeyTableItems = () => {
    let items = [];

    userProfileContext.apiKeys.forEach((apiKey) => {
      items.push({
        key: apiKey.Id,
        name: apiKey.Name,
        token: apiKey.Token,
        usageCount: apiKey.UsageCount,
        createdAt: FormatDatetime(apiKey.CreatedAt),
        lastUsed: FormatDatetime(apiKey.LastUsed),
      });
    });

    return items;
  };

  const getSessionTableItems = () => {
    let items = [];

    userProfileContext.sessions.sort(
      (a, b) => b.ConnectionStatus - a.ConnectionStatus
    );

    userProfileContext.sessions.forEach((session) => {
      items.push({
        key: session.IdForDeletion,
        userAgent: session.UserAgent,
        connectionStatus: getConnectionStatusItem(session.ConnectionStatus),
        lastUsed: FormatDatetime(session.LastUsed),
        expiresAt: FormatDatetime(session.ExpiresAt),
      });
    });

    return items;
  };

  const beforeUpload = (file) => {
    if (file.size > Constants.MAX_AVATAR_SIZE) {
      notificationApi["error"]({
        message: t("userProfile.changeAvatarError.notification.message"),
        description: t(
          "userProfile.changeAvatarError.notification.description",
          { MAX_AVATAR_SIZE: Constants.MAX_AVATAR_SIZE / 1024 / 1024 }
        ),
      });
      return false;
    }
    return true;
  };

  const isButtonDisabled = () => {
    if (
      !isEmailValid(email) ||
      sideBarContext.username.length < Constants.GLOBALS.MIN_USERNAME_LENGTH
    ) {
      return true;
    }

    if (
      username !== sideBarContext.username ||
      email !== userProfileContext.email ||
      (oldPassword !== "" &&
        newPassword !== "" &&
        newPassword === repeatedNewPassword)
    ) {
      return false;
    }

    return true;
  };

  const handleOnSubmit = () => {
    const changes = {};

    if (sideBarContext.username !== username) {
      changes.username = username;
    }

    if (userProfileContext.email !== email) {
      changes.email = email;
    }

    if (
      oldPassword !== "" &&
      newPassword !== "" &&
      newPassword === repeatedNewPassword
    ) {
      changes.oldPassword = EncodeStringToBase64(oldPassword);
      changes.newPassword = EncodeStringToBase64(newPassword);
    }

    webSocketContext.SendSocketMessage(SentMessagesCommands.UpdateUserProfile, {
      changes: changes,
    });
  };

  const onCreateNewApiKeyConfirm = () => {
    webSocketContext.SendSocketMessage(
      SentMessagesCommands.CreateNewUserApiKey,
      { Name: newApiKeyName }
    );

    setNewApikeyName("");
  };

  const telegramVerifyStatusRequest = (updateOnlyOnVerified) =>
    myFetch(
      `/status/${appContext.userId.current}`,
      "GET",
      null,
      {},
      myFetchContentType.JSON,
      Constants.TELEGRAM_BOT_MANAGER_ADDRESS
    )
      .then((data) => {
        // manual check verify status by clicking reload icon
        if (!updateOnlyOnVerified) {
          setTelegramNotificationSubscribedStatus(data.Status);
        }

        // used for auto check verify status
        if (data.Status === true) {
          setTelegramNotificationSubscribedStatus(data.Status);
          clearInterval(statusInterval.current);
          telegramVerifyCodeRequestTime.current = null;
        }
      })
      .catch(() =>
        notificationApi["error"]({
          message: t("userProfile.telegram.connectionFailedPopover.title"),
          description: t("common.contactAdmin"),
        })
      );

  const TelegramNotificationSubscribedStatusItem = () => {
    // check verified status
    if (telegramNotificationSubscribedStatus === null) {
      return (
        <Button
          style={{ width: "100%" }}
          onClick={() => telegramVerifyStatusRequest()}
        >
          {t("userProfile.telegram.checkStatusButton")}
        </Button>
      );
    }

    // unsubscribe
    if (telegramNotificationSubscribedStatus === true) {
      return (
        <Space>
          <span>{t("userProfile.telegram.subscribed")}</span>

          <Popconfirm
            title={t("userProfile.telegram.disconnectPopconfirm.title")}
            description={t(
              "userProfile.telegram.disconnectPopconfirm.description"
            )}
            okText={t("common.button.confirm")}
            cancelText={t("common.button.cancel")}
            onConfirm={() => {
              myFetch(
                `/verifycode/${appContext.userId.current}`,
                "DELETE",
                null,
                {},
                myFetchContentType.JSON,
                Constants.TELEGRAM_BOT_MANAGER_ADDRESS
              ).then(() => {
                setTelegramNotificationSubscribedStatus(null);
              });
            }}
          >
            <DisconnectOutlined />
          </Popconfirm>
        </Space>
      );
    }

    // subscribe
    if (telegramNotificationSubscribedStatus === false) {
      return (
        <Button
          icon={<NotificationOutlined />}
          style={{ width: "100%" }}
          onClick={() =>
            myFetch(
              `/verifycode/${appContext.userId.current}`,
              "GET",
              null,
              {},
              myFetchContentType.JSON,
              Constants.TELEGRAM_BOT_MANAGER_ADDRESS
            ).then((data) => {
              setTelegramNotificationSubscribedStatus(data.Code);
            })
          }
        >
          {t("userProfile.telegram.subscribeButton")}
        </Button>
      );
    }

    // verification code
    return (
      <Space>
        <Popover
          placement="left"
          title={t("userProfile.telegram.verificationPopover.title")}
          overlayStyle={{ maxWidth: 300 }}
          content={
            <Space direction="vertical">
              <Trans
                i18nKey="userProfile.telegram.verificationPopover.description"
                values={{
                  command: `<strong>/verify ${telegramNotificationSubscribedStatus}</strong>`,
                }}
              />

              <div style={{ textAlign: "center" }}>
                <img
                  src={`${Constants.TELEGRAM_BOT_MANAGER_CONTENT_ADDRESS}img/qrcode.png`}
                  width={200}
                />
              </div>
            </Space>
          }
        >
          <InfoCircleOutlined style={{ color: Constants.COLORS.ICON_INFO }} />
        </Popover>

        <span>{telegramNotificationSubscribedStatus}</span>

        <MyCopyIcon
          text={`/verify ${telegramNotificationSubscribedStatus}`}
          notificationApi={notificationApi}
        />

        <MyReloadIcon onClick={() => telegramVerifyStatusRequest()} />
      </Space>
    );
  };

  useEffect(() => {
    if (
      telegramNotificationSubscribedStatus !== true &&
      telegramNotificationSubscribedStatus !== false &&
      telegramNotificationSubscribedStatus !== null
    ) {
      telegramVerifyCodeRequestTime.current = new Date();

      statusInterval.current = setInterval(() => {
        // check expiration time
        if (
          telegramVerifyCodeRequestTime.current !== null &&
          new Date() - telegramVerifyCodeRequestTime.current >
            Constants.GLOBALS.TELEGRAM_VERIFY_CODE_EXPIRATION_TIME
        ) {
          clearInterval(statusInterval.current);
          telegramVerifyCodeRequestTime.current = null;
          setTelegramNotificationSubscribedStatus(false);
          return;
        }

        telegramVerifyStatusRequest(true);
      }, 1000);
    }

    return () => clearInterval(statusInterval.current);
  }, [telegramNotificationSubscribedStatus]);

  useEffect(() => {
    const userProfileRequest = () =>
      myFetch("/user/profile", "GET").then((data) => {
        userProfileContext.setEmail(data.Email);
        userProfileContext.setSessions(data.Sessions);
        userProfileContext.setApiKeys(data.ApiKeys);

        setEmail(data.Email);
      });

    userProfileRequest();

    const handleUserProfileRequest = () => userProfileRequest();

    document.addEventListener(
      wsConnectionCustomEventName,
      handleUserProfileRequest
    );

    return () =>
      document.removeEventListener(
        wsConnectionCustomEventName,
        handleUserProfileRequest
      );
  }, []);

  useEffect(
    () => setUsername(sideBarContext.username),
    [sideBarContext.username]
  );

  useEffect(
    () => setEmail(userProfileContext.email),
    [userProfileContext.email]
  );

  return (
    <>
      {notificationContextHolder}

      <Typography.Title level={4}>
        {t("userProfile.header.yourProfile")}
      </Typography.Title>

      <Card>
        <Form layout="vertical">
          <Row>
            <Col xs={24} md={4}>
              <Upload
                accept={Constants.ACCEPTED_AVATAR_FILE_TYPES.join(",")}
                action={Constants.API_ADDRESS + "/user/avatar"}
                maxCount={1}
                showUploadList={false}
                beforeUpload={beforeUpload}
                headers={{
                  "X-Authorization": getUserSessionFromLocalStorage(),
                }}
              >
                <MyUserAvatar avatar={sideBarContext.avatar} size={256} />
              </Upload>
            </Col>

            <Col xs={24} md={{ span: 6, offset: 14 }}>
              <Form.Item label={t("userProfile.form.language")}>
                <Select
                  style={{ width: "100%" }}
                  defaultValue={i18n.language}
                  options={[
                    {
                      value: "en",
                      label: "English",
                    },
                    {
                      value: "de",
                      label: "Deutsch",
                    },
                  ]}
                  onChange={(e) => i18n.changeLanguage(e)}
                />
              </Form.Item>

              <Form.Item label={t("userProfile.telegram.title")}>
                <TelegramNotificationSubscribedStatusItem />
              </Form.Item>

              <Form.Item label={t("userProfile.button.logout.title")}>
                <Popconfirm
                  title={t("userProfile.button.logout.popconfirm.title")}
                  description={t(
                    "userProfile.button.logout.popconfirm.description"
                  )}
                  okText={t("common.button.confirm")}
                  cancelText={t("common.button.cancel")}
                  onConfirm={() => {
                    setUserSession();
                    window.location.href = "/";

                    fetch(`${Constants.API_ADDRESS}/user/auth/logout`, {
                      method: "DELETE",
                      headers: {
                        "Content-Type": "application/json",
                        "X-Authorization": userSession,
                      },
                    }).catch(console.error);
                  }}
                >
                  <Button block icon={<LogoutOutlined />}>
                    {t("userProfile.button.logout.title")}
                  </Button>
                </Popconfirm>
              </Form.Item>
            </Col>
          </Row>

          <Form.Item
            label={t("userProfile.form.username")}
            hasFeedback
            validateStatus={
              username < Constants.GLOBALS.MIN_USERNAME_LENGTH && "error"
            }
          >
            <Input
              value={username}
              onChange={(e) => setUsername(e.target.value)}
              minLength={Constants.GLOBALS.MIN_USERNAME_LENGTH}
              maxLength={Constants.GLOBALS.MAX_USERNAME_LENGTH}
            />
          </Form.Item>
          <Form.Item
            label={t("userProfile.form.email")}
            hasFeedback
            validateStatus={!isEmailValid(userProfileContext.email) && "error"}
          >
            <Input
              type="email"
              value={email}
              onChange={(e) => setEmail(e.target.value)}
            />
          </Form.Item>
          <Form.Item
            label={t("userProfile.form.oldPassword")}
            hasFeedback
            validateStatus={
              oldPassword !== "" &&
              oldPassword.length < Constants.GLOBALS.MIN_PASSWORD_LENGTH &&
              "error"
            }
          >
            <Input.Password
              value={oldPassword}
              onChange={(e) => setOldPassword(e.target.value)}
              minLength={Constants.GLOBALS.MIN_PASSWORD_LENGTH}
              maxLength={Constants.GLOBALS.MAX_PASSWORD_LENGTH}
            />
          </Form.Item>
          <Form.Item
            label={t("userProfile.form.newPassword")}
            hasFeedback
            validateStatus={
              newPassword !== "" &&
              newPassword.length < Constants.GLOBALS.MIN_PASSWORD_LENGTH &&
              "error"
            }
          >
            <Input.Password
              value={newPassword}
              onChange={(e) => setNewPassword(e.target.value)}
              minLength={Constants.GLOBALS.MIN_PASSWORD_LENGTH}
              maxLength={Constants.GLOBALS.MAX_PASSWORD_LENGTH}
            />
          </Form.Item>
          <Form.Item
            label={t("userProfile.form.repeatNewPassword")}
            hasFeedback
            validateStatus={newPassword !== repeatedNewPassword && "error"}
          >
            <Input.Password
              value={repeatedNewPassword}
              onChange={(e) => setRepeatedNewPassword(e.target.value)}
              minLength={Constants.GLOBALS.MIN_PASSWORD_LENGTH}
              maxLength={Constants.GLOBALS.MAX_PASSWORD_LENGTH}
            />
          </Form.Item>
          <Form.Item style={{ margin: 0 }}>
            <Button
              type="primary"
              htmlType="submit"
              onClick={() => handleOnSubmit()}
              disabled={isButtonDisabled()}
            >
              {t("common.button.save")}
            </Button>
          </Form.Item>
        </Form>
      </Card>

      <br />

      <Typography.Title level={4}>
        {t("userProfile.header.yourSessions")} (
        {userProfileContext.sessions.length})
      </Typography.Title>

      <Table
        scroll={{ x: "max-content" }}
        columns={getSessionTableColumns()}
        dataSource={getSessionTableItems()}
      />

      {hasPermission(
        appContext.userPermissions,
        Constants.PERMISSIONS.USER_PROFILE.API_KEYS
      ) && (
        <>
          {" "}
          <div
            style={{
              display: "flex",
              justifyContent: "space-between",
            }}
          >
            <Typography.Title level={4}>
              {t("userProfile.header.yourApiKeys")} (
              {userProfileContext.apiKeys.length}){" "}
              <Tooltip title={t("userProfile.icon.viewApiDoc")}>
                <Link
                  target="_blank"
                  to={Constants.STATIC_CONTENT_ADDRESS + "swagger/index.html"}
                >
                  <FileTextOutlined style={{ fontSize: 16 }} />
                </Link>
              </Tooltip>
            </Typography.Title>

            <Popconfirm
              placement="topRight"
              title={t("userProfile.button.createApiKey.popconfirm.title")}
              description={
                <Input
                  placeholder="Name"
                  value={newApiKeyName}
                  onChange={(e) => setNewApikeyName(e.target.value)}
                  maxLength={Constants.GLOBALS.MAX_USER_API_KEY_NAME_LENGTH}
                />
              }
              okButtonProps={{
                disabled:
                  newApiKeyName.length <
                    Constants.GLOBALS.MIN_USER_API_KEY_NAME_LENGTH ||
                  newApiKeyName.length >
                    Constants.GLOBALS.MAX_USER_API_KEY_NAME_LENGTH,
              }}
              okText={t("userProfile.button.createApiKey.popconfirm.okText")}
              cancelText={t("common.button.cancel")}
              onConfirm={() => onCreateNewApiKeyConfirm()}
            >
              <Button icon={<KeyOutlined />}>
                {t("userProfile.button.createApiKey.title")}
              </Button>
            </Popconfirm>
          </div>
          <Table
            scroll={{ x: "max-content" }}
            columns={getApiKeyTableColumns()}
            dataSource={getApiKeyTableItems()}
          />
        </>
      )}
    </>
  );
}
